#!/bin/sh
#
# Copyright (c) Edson Branti
# 
#$Id: mountdisks.sh,v 1.1 2005/12/24 17:28:47 rionda Exp $

###############################################
# We will try to detect a set of disks and 
# existing slices and try to mount every 
# FAT16/32, NTFS, UFS, EXT2 partition we find.
###############################################

DRIVER="ext2fs msdos ntfs"
MOUNT_OPTION=
EXT_LOGICAL="5 6 7 8"
UFS_PART=1
MSDOS_PRIMARY=1
NTFS_PRIMARY=1
EXT2FS_PRIMARY=1

MSDOS_LOGICAL=1
NTFS_LOGICAL=1
EXT2FS_LOGICAL=1

UFS_TOO_MANY=0
MSDOS_TOO_MANY=0
NTFS_TOO_MANY=0
EXT2FS_TOO_MANY=0

OPTION=`echo $@`
if [ "${OPTION}" != "rw" -a "${OPTION}" != "ro" ]; then
	echo "Program: mount_disks.sh"
	echo "Parameters:" 
	echo "   - ro mounts partitions in READ ONLY mode;"
	echo "   - rw mounts partitions in READ/WRITE mode."
	exit 1
fi
if [ "${OPTION}" = "rw" ]; then
	MOUNT_OPTION="-r"
fi
for i in `egrep "^(ad|ar|da|cd|acd)[0-9]:" /var/run/dmesg.boot | cut -d':' -f1 | sort -u`
do
    a=1
    for x in `fdisk /dev/$i | awk /sysid/'{print $2}'`
    do
        case ${x} in
        1|4|6|11|12|13)
            echo "FAT16/32 partition found at /dev/${i}s${a}, mounting it under /mnt/dos.${MSDOS_PRIMARY} ..."
            mount -t msdos ${MOUNT_OPTION} /dev/${i}s${a} /mnt/dos.${MSDOS_PRIMARY} >/dev/null 2>&1
            MSDOS_PRIMARY=`expr $MSDOS_PRIMARY + 1`
            ;;
        5|15)
            echo "Extended partition found, trying to detect and mount logical partitions ..."
            for y in $EXT_LOGICAL
            do
                for w in $DRIVER
                do
                    mount -t $w ${MOUNT_OPTION} /dev/${i}s${y} /mnt/tmp >/dev/null 2>&1
                    RETVAL=$?
                    if [ $RETVAL = 0 ]; then
                        umount /mnt/tmp
                        if [ $w = "msdos" ]; then
                            if [ $MSDOS_LOGICAL -lt 5 ]; then
                                echo "FAT32 partition found at /dev/${i}s${y}, mounting it under /mnt/dos.ext.${MSDOS_LOGICAL} ..."
                                mount -t msdos ${MOUNT_OPTION} /dev/${i}s${y} /mnt/dos.ext.${MSDOS_LOGICAL} >/dev/null 2>&1
                                MSDOS_LOGICAL=`expr $MSDOS_LOGICAL + 1`
                            else
                                if [ $MSDOS_TOO_MANY -eq 0 ]; then
                                    echo "Too many msdos logical partitions: only 4 can be mounted simultaneously."
                                    MSDOS_TOO_MANY=1
                                fi
                            fi
                        fi
                        if [ $w = "ntfs" ]; then
                            if [ $NTFS_LOGICAL -lt 5 ]; then
                                echo "NTFS partition found at /dev/${i}s${y}, mounting it under /mnt/ntfs.ext.${NTFS_LOGICAL} ..."
                                mount -t ntfs /dev/${i}s${y} /mnt/ntfs.ext.${NTFS_LOGICAL} >/dev/null 2>&1
                                NTFS_LOGICAL=`expr $NTFS_LOGICAL + 1`
                            else
                                if [ $NTFS_TOO_MANY -eq 0 ]; then
                                    echo "Too many ntfs logical partitions: only 4 can be mounted simultaneously."
                                    NTFS_TOO_MANY=1
                                fi
                            fi
                        fi
                        if [ $w = "ext2fs" ]; then
                            if [ $EXT2FS_LOGICAL -lt 5 ]; then
                                echo "EXT2FS partition found at /dev/${i}s${y}, mounting it under /mnt/ext2fs.ext.${EXT2FS_LOGICAL} ..."
                                mount -t ext2fs ${MOUNT_OPTION} /dev/${i}s${y} /mnt/ext2fs.ext.${EXT2FS_LOGICAL} >/dev/null 2>&1
                                EXT2FS_LOGICAL=`expr $EXT2FS_LOGICAL + 1`
                            else
                                if [ $EXT2FS_TOO_MANY -eq 0 ]; then
                                    echo "Too many ext2fs logical partitions: only 4 can be mounted simultaneously."
                                    EXT2FS_TOO_MANY=1
                                fi
                            fi
                        fi
                    fi
                done
            done
            ;;
        7)
            echo "NTFS partition found at /dev/${i}s${a}, mounting it under /mnt/ntfs.${NTFS_PRIMARY} ..."
            mount -t ntfs /dev/${i}s${a} /mnt/ntfs.${NTFS_PRIMARY} >/dev/null 2>&1
            NTFS_PRIMARY=`expr $NTFS_PRIMARY + 1`
            ;;
        131)
            echo "EXT2FS partition found at /dev/${i}s${a}, mounting it under /mnt/ext2fs.${EXT2FS_PRIMARY} ..."
            mount -t ext2fs ${MOUNT_OPTION} /dev/${i}s${a} /mnt/ext2fs.${EXT2FS_PRIMARY} >/dev/null 2>&1
            EXT2FS_PRIMARY=`expr $EXT2FS_PRIMARY + 1`
            ;;
        165)
            echo "UFS partition found at /dev/${i}s${a}, detecting slices..."
            for j in `disklabel -r /dev/${i}s${a} 2>/dev/null | grep BSD | awk '{print substr($1,1,length($1)-1)}'`
            do
                if [ $UFS_PART -lt 9 ]; then
                    echo "UFS slice found at /dev/${i}s${a}${j}, mounting it under /mnt/ufs.${UFS_PART}"
                    mount ${MOUNT_OPTION} /dev/${i}s${a}${j} /mnt/ufs.${UFS_PART} >/dev/null 2>&1
                    UFS_PART=`expr ${UFS_PART} + 1`
                else
                    if [ $UFS_TOO_MANY -eq 0 ]; then
                        echo "Too many slices found, only 8 can be mounted simultaneously."    
                        UFS_TOO_MANY=1
                    fi
                fi
            done
            ;;
        esac
        a=`expr $a + 1`
    done
done
